c     Copyright (C)
c     2000 -2013 Universite Paul Sabatier,
c     2000 -2013 Centre National de la Recherche Scientifique
c     2000 -2013 Ecole des Mines d'Albi
c     2014-2018 Msostar
c
c     This program is free software; you can redistribute it and/or modify
c     it under the terms of the GNU General Public License as published by
c     the Free Software Foundation; either version 2, or (at your option)
c     any later version.

c     This program is distributed in the hope that it will be useful,
c     but WITHOUT ANY WARRANTY; without even the implied warranty of
c     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
c     GNU General Public License for more details.

c     You should have received a copy of the GNU General Public License
c     along with this program; if not, write to the Free Software
c     Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
c     USA.
c
      double precision function Planck(T,lambda1,lambda2)

      implicit none

      double precision in,c1,c2,T,lambda_c,lambda1,lambda2,B
      double precision intB,w1,w2
      integer Nb,Nq,band,quad
      real PLKAVG

c     Cette function calcule la luminance de Planck  une valeur de
c     la temprature et de la longueur d'onde
c
c     Entres :
c       + T : temprature en Kelvin
c       + lambda1 : longueur d'onde infrieure en micromtres
c       + lambda2 : longueur d'onde suprieure en micromtres
c
c     Sorties :
c       + Planck : valeur de la luminance du corps noir en W/m2/sr/m
c                  moyenne sur l'intervalle [lambda1,lambda2]
c

      include 'max.inc'
      include 'data.inc'
      include 'options.inc'
      include 'param.inc'

      lambda_c=(lambda1+lambda2)/2.0D+0
c     Si on demande de prendre la valeur de Planck au centre
c     de la bande troite
      if (cpb.eq.1) then
         Planck=B(T,lambda_c)
c     Sinon on demande d'intgrer Planck sur la bande troite
      else
         w1=1.0D+4/lambda2
         w2=1.0D+4/lambda1
         Planck=dble(PLKAVG(sngl(w1),sngl(w2),sngl(T)))
      endif


      return
      end




      function B(T,lambda)
      implicit none
c     
c     Luminance de Planck en une valeur de la longueur d'onde
c     et de la temprature
c
      double precision B,T,lambda
      double precision c1,c2,in
      include 'param.inc'

      c1=2*pi*hPl*c0**2 ! W.m2
      c2=hPl*c0/kBz*1.D+6 ! m.K
      in=lambda*T
      if (T.eq.0) then
         B=0.0D+0
      else
         B=c1/(in**5*(dexp(c2/in)-1))*T**5*1.D+24/pi ! W/m2/sr/m
      endif

      return
      end



      double precision function planck_int_wavnb(nu_lo,nu_hi,T)
      implicit none
      include 'max.inc'
c
c     Purpose: to compute the Planck function, integrated between two
c     values of the wavenumber
c
c     Inputs:
c       + nu_lo: lower value of wavenumber [cm^{-1}]
c       + nu_hi: higer value of wavenumber [cm^{-1}]
c       + T: temperature [K]
c
c     Output:
c       + planck_int_wavnb: Planck function at temperature T, integrated between
c            nu_lo and nu_hi [W//m/sr]
c

c     inputs
      double precision nu_lo,nu_hi,T
c     temp
      real PLKAVG
c     label
      integer strlen
      character*(Nchar_mx) label
      label='subroutine planck_int_wavnb'

      if (nu_hi.lt.nu_lo) then
         call error(label)
         write(*,*) 'Incorrect input parameters:'
         write(*,*) 'nu_lo=',nu_lo,' > nu_hi=',nu_hi
         stop
      endif

      if (T.lt.0.0D+0) then
         call error(label)
         write(*,*) 'Incorrect input parameter:'
         write(*,*) 'T=',T
         stop
      endif

      planck_int_wavnb=PLKAVG(real(nu_lo),real(nu_hi),real(T)) ! W/m/sr

      return
      end



      double precision function planck_int_wavlng(lambda_lo,lambda_hi,
     &     T)
      implicit none
      include 'max.inc'
c
c     Purpose: to compute the Planck function, integrated between two
c     values of the wavelength
c
c     Inputs:
c       + lambda_lo: lower value of wavelength [m]
c       + lambda_hi: higer value of wavelength [m]
c       + T: temperature [K]
c
c     Output:
c       + planck_int_wavlng: Planck function at temperature T, integrated between
c            lambda_lo and lambda_hi [W//m/sr]
c

c     inputs
      double precision lambda_lo,lambda_hi,T
c     temp
      double precision planck_int_wavnb
      double precision nu_lo,nu_hi
c     label
      integer strlen
      character*(Nchar_mx) label
      label='function planck_int_wavlng'

      if (lambda_hi.lt.lambda_lo) then
         call error(label)
         write(*,*) 'Incorrect input parameters:'
         write(*,*) 'lambda_lo=',lambda_lo,' > lambda_hi=',lambda_hi
         stop
      endif

      if (T.lt.0.0D+0) then
         call error(label)
         write(*,*) 'Incorrect input parameter:'
         write(*,*) 'T=',T
         stop
      endif

      call lambda2nu(lambda_lo,nu_hi)
      call lambda2nu(lambda_hi,nu_lo)
      planck_int_wavlng=planck_int_wavnb(nu_lo,nu_hi,T) ! W/m/sr

      return
      end



      subroutine nu2lambda(nu,lambda)
      implicit none
      include 'max.inc'
c
c     Purpose: to convert a wavenumber (cm) into a wavelength (m)
c
c     I/O
      double precision nu,lambda
c     temp
      integer strlen
      character*(Nchar_mx) label
      label='subroutine nu2lambda'

      if (nu.le.0.0D+0) then
         call error(label)
         write(*,*) 'Bad input parameter:'
         write(*,*) 'nu=',nu
         stop
      else
         lambda=1.0D+4/nu
      endif

      return
      end



      subroutine lambda2nu(lambda,nu)
      implicit none
      include 'max.inc'
c
c     Purpose: to convert a wavelength (m) into a wavenumber (cm)
c
c     I/O
      double precision lambda,nu
c     temp
c     label
      integer strlen
      character*(Nchar_mx) label
      label='subroutine lambda2nu'

      if (lambda.le.0.0D+0) then
         call error(label)
         write(*,*) 'Bad input parameter:'
         write(*,*) 'lambda=',lambda
         stop
      else
         nu=1.0D+4/lambda
      endif

      return
      end
